<?php

    /*

        Yellow Duck Framework version 2.0
        (c) Copyright 2002-2005 Pieter Claerhout

        This library is free software; you can redistribute it and/or
        modify it under the terms of the GNU Lesser General Public
        License as published by the Free Software Foundation; either
        version 2.1 of the License, or (at your option) any later version.

        This library is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
        Lesser General Public License for more details.

        You should have received a copy of the GNU Lesser General Public
        License along with this library; if not, write to the Free Software
        Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    */

    // Check if the framework is loaded
    if ( ! defined( 'YD_FW_NAME' ) ) {
        die( 'Yellow Duck Framework is not loaded.' );
    }

    // Includes
    include_once( YD_DIR_HOME_CLS . '/YDForm.php');

    /**
     *        This is the class that define a button form element.
     */
    class YDFormElement_Link extends YDFormElement {

        /**
         *        This is the class constructor for the YDFormElement_Button class.
         *
         *        @param $form                The name of the form to which this element is connected.
         *        @param $name                The name of the form element.
         *        @param $label                (optional) The label for the form element.
         *        @param $attributes        (optional) The attributes for the form element.
         *        @param $options                (optional) The options for the elment.
         */
        function YDFormElement_Link( $form, $name, $label='', $attributes=array(), $options=array() ) {

            // Initialize the parent
            $this->YDFormElement( $form, $name, $label, $attributes, $options );

            // Set the type
            $this->_type = 'link';

            if ( is_string( $options ) && strlen( $options ) ) {
                $link = $options;
            } else if ( isset( $attributes['href'] ) ) {
                $link = $attributes['href'];
            } else {
                $link = $label;
            }
            
            $this->_attributes['href'] = $link;
            
            $this->setValue( $label );
            $this->_label = '';
            $this->_placeLabel = 'none';

            // Indicate if filters need to be applied
            $this->_applyFilters = false;

            // Indicate we are a button type
            $this->_isButton = false;

        }

        /**
         *      This function returns a boolean indicating if the element value was
         *      modified from it's default value.
         *
         *      @returns        Boolean indicating if the element was modified.
         */
        function isModified() {
            return false;
        }

        /**
         *        This function will return the element as HTML.
         *
         *        @returns        The form element as HTML text.
         */
        function toHtml() {

            // Get the HTML
            return '<a' . YDForm::_convertToHtmlAttrib( $this->_attributes ) . '>' . $this->_value . '</a>';

        }


        /**
         *	This function returns the default javascript event of this element
         */
        function getJSEvent(){ 

            return 'onclick';
        }


    }

?>